use tokio::net::TcpStream;
use std::io;
use domo_proto::packet::{Packet};
use log::trace;
use tokio::io::{AsyncReadExt};
use domo_proto::packet::identifier::Identifier;
use domo_proto::packet::packet_data::PacketData;

pub mod client;

pub async fn from_tcp_stream(stream: &mut TcpStream) -> io::Result<Packet> {
    let version = stream.read_u8().await?;
    match version {
        0x01 => {
            // Example: Read packet data and create a Packet::V1 instance
            let src = Identifier::from(stream.read_u32().await?); // Assuming you have a similar function for Identifier
            let dest = Identifier::from(stream.read_u32().await?); // Assuming you have a similar function for Identifier
            let packet_id = Identifier::from(stream.read_u32().await?); // Assuming you have a similar function for Identifier
            let reply_to = Identifier::from(stream.read_u32().await?); // Assuming you have a similar function for Identifier
            let command = stream.read_u8().await?; // Read u8 directly
            let data_length = stream.read_u16().await?; // Read u16 directly

            trace!("received => src: {src}, dest: {dest}, packet_id: {packet_id}, reply_to: {reply_to}, command: {command}, data_length: {data_length}");

            let mut data = vec![0u8; data_length as usize];
            stream.read_exact(&mut data).await?;

            // Create a Packet::V1 instance
            let packet = Packet {
                src,
                dest,
                packet_id,
                reply_to,
                command,
                data: PacketData::new(data), // Assuming PacketData has a constructor
            };

            if packet.get_crc32() == stream.read_u32().await? {
                Ok(packet)
            } else {
                Err(io::Error::new(io::ErrorKind::InvalidData, "Validation failed"))
            }
        }
        _ => Err(io::Error::new(io::ErrorKind::InvalidData, format!("received invalid data; version is {version}")))
    }
}
